//+------------------------------------------------------------------+
//|                                                    SmWPR_WPR.mq4 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Gold
#property indicator_color2 Blue

//---- input parameters
extern int ExtWPRPeriod=14;
extern int  T3_Period = 8;  
extern double b = 0.7;
extern int psel=-15;
extern int pbuy=-85;

//---- buffers
double ExtWPRBuffer[];
double FWPR[];
double SmWPRBuffer[];
double TrigBuffer[];

double e1, e2, e3, e4, e5, e6;
double n, c1, c2, c3, c4, w1, w2, b2, b3;
//----
int time2; double E1, E2, E3, E4, E5, E6;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   string sShortName;
   IndicatorBuffers(4);
//----
   SetIndexBuffer(0,SmWPRBuffer);
   SetIndexBuffer(1,TrigBuffer);
   SetIndexBuffer(2,FWPR);
   SetIndexBuffer(3,ExtWPRBuffer);
//----
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(1,DRAW_LINE, STYLE_DOT);
//----
   sShortName="SmWPR_WPR";
   IndicatorShortName(sShortName);
   SetIndexLabel(0,sShortName);
   SetIndexLabel(1,"Trigger");
//----
   SetIndexDrawBegin(0,ExtWPRPeriod+T3_Period);
   SetIndexDrawBegin(1,ExtWPRPeriod+T3_Period);
//----   
   b2 = b*b;
   b3 = b2*b;
   c1 = -b3;
   c2 = (3*(b2 + b3));
   c3 = -3*(2*b2 + b + b3);
   c4 = (1 + 3*b + b3 + 3*b2);
   if(T3_Period < 1) 
       T3_Period = 1;
   n = 1 + 0.5*(T3_Period - 1);
   w1 = 2 / (n + 1);
   w2 = 1 - w1;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int i,limit,nCountedBars;
   double e1, e2, e3, e4, e5, e6;  
//----
   if(Bars<=ExtWPRPeriod) return(0);
//----
   nCountedBars=IndicatorCounted();
//----
   i=Bars-ExtWPRPeriod-1;
   if(nCountedBars>ExtWPRPeriod) 
      i=Bars-nCountedBars-1;
//----      
   if(nCountedBars>0) limit=Bars-nCountedBars; 
   else limit=Bars-1; 
//----     
   for(int j=limit; j>=0; j--)
      FWPR[j]=iWPR(NULL, 0, ExtWPRPeriod, j); 
//----          
   while(i>=0)
     {
      int n, sn, r, result;
      for(n=1, r=i; n<ExtWPRPeriod; n++)
       {
        sn=i+n;
        if(FWPR[r]>FWPR[sn]) result=r;
        else result=sn;
        r=result;
       }
      double dMaxHigh=FWPR[r];     
      for(n=1, r=i; n<ExtWPRPeriod; n++)
       {
        sn=i+n;
        if(FWPR[r]<FWPR[sn]) result=r;
        else result=sn;
        r=result;
       }
      double dMinLow=FWPR[r]; 
      ExtWPRBuffer[i]=-100*(dMaxHigh-FWPR[i])/(dMaxHigh-dMinLow);
      i--;
     }
//----
   if(Bars-1 < T3_Period)
       return(0);
//----
   int MaxBar, counted_bars = IndicatorCounted();
//----
   if(counted_bars < 0)
       return(-1);
//----
   if(counted_bars > 0) 
       counted_bars--;
//----
   MaxBar = Bars - 1 - T3_Period - ExtWPRPeriod;
//----
   limit = (Bars - 1 - counted_bars);
//----
   if(limit > MaxBar)
     {
       for(int bar = Bars - 1; bar >= MaxBar; bar--)
           SmWPRBuffer[bar] = 0.0;
       limit = MaxBar; 
     }
//----
   int Tnew = Time[limit+1];
   if(limit < MaxBar)
   if(Tnew == time2)
     {
       e1 = E1; 
       e2 = E2; 
       e3 = E3; 
       e4 = E4; 
       e5 = E5; 
       e6 = E6;  
     }
   else 
     {
       if(Tnew > time2)
           Print("ERROR01");
       else 
           Print("ERROR02");
       return(-1);
     }
//----
   for(bar = limit; bar >= 0; bar--)
     {
       if(bar == 1)
           if(((limit == 1)&&(time2 != Time[2])) || (limit > 1))
             {
               time2 = Time[2];
               E1 = e1; 
               E2 = e2; 
               E3 = e3; 
               E4 = e4; 
               E5 = e5; 
               E6 = e6;
             }
       e1 = w1*ExtWPRBuffer[bar] + w2*e1;
       e2 = w1*e1 + w2*e2;
       e3 = w1*e2 + w2*e3;
       e4 = w1*e3 + w2*e4;
       e5 = w1*e4 + w2*e5;
       e6 = w1*e5 + w2*e6;
       SmWPRBuffer[bar]=c1*e6 + c2*e5 + c3*e4 + c4*e3;
     }   
//----
   for(j=limit; j>=0; j--)
    {
     if (SmWPRBuffer[j]>=-50) TrigBuffer[j]=psel; 
     if (SmWPRBuffer[j]<-50) TrigBuffer[j]=pbuy;
    } 
//----
   return(0);
  }
//+------------------------------------------------------------------+